************************************************************************
* sanc_cc_v1.40 package.
************************************************************************
* File (cc_ha_2122_1314.f) is created on Tue Aug  9 22:44:04 MSD 2011.
************************************************************************
* This is the FORTRAN module (cc_ha_2122_1314) to calculate differential
* hard photon Bremsstrahlung for the anti-tp -> anti-bt + anti-up + dn decay.
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2011.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine cc_ha_2122_1314 (s,spr,ph4,costh4,costh5,hard)
      implicit none!
      include 's2n_declare.h'

      real*8 sq,sqspr,qw
      real*8 p1p2,p1p3,p1p4,p1p5,p2p3,p2p4,p2p5,p3p4,p3p5,p4p5
      real*8 ph4,cosph4,sinph4,costh4,sinth4,costh5,sinth5
      real*8 sqrtltpbts,isqrtltpbts,sqrtlsprupdn,isqrtlsprupdn
      complex*16 propwspr,propiwspr,propwsprc,propws,propiws,propwsc
      real*8 iz1bt,iz2tp,iz3up,iz4dn
      complex*16 identp25m,dentp25m,dentp25mc

      cmw2 = mw2
      qw = 1d0

      sq = dsqrt(s)
      sqspr = dsqrt(spr)

      cosph4 = dcos(ph4)
      sinph4 = dsin(ph4)
      sinth4 = dsqrt(1d0-costh4**2)
      sinth5 = dsqrt(1d0-costh5**2)

      if (irun.eq.0) then
         propiwspr = (cmw2-spr)
      elseif (irun.eq.1) then
         propiwspr = (dcmplx(mw**2,-spr*ww/mw)-spr)
      endif
      propwspr = 1d0/propiwspr
      propwsprc = dconjg(propwspr)

      if (irun.eq.0) then
         propiws = (cmw2-s)
      elseif (irun.eq.1) then
         propiws = (dcmplx(mw**2,-s*ww/mw)-s)
      endif
      propws = 1d0/propiws
      propwsc = dconjg(propws)

      sqrtltpbts = sqrt(rtpm2**2-2d0*rtpm2*(rbtm2+s)+(rbtm2-s)**2)
      isqrtltpbts = 1d0/sqrtltpbts
      sqrtlsprupdn = sqrt(spr**2-2d0*spr*(rupm2+rdnm2)+(rupm2-rdnm2)**2)
      isqrtlsprupdn = 1d0/sqrtlsprupdn

      p1p2=
     & +1d0/2*s-1d0/2*mbt**2-1d0/2*mtp**2

      p1p3=
     & +sqrtltpbts*sinph4*sinth4*sinth5*(1d0/4*sqspr/sq)
     & +sqrtltpbts*costh4*costh5*(-1d0/8-1d0/8/s*spr)
     & +sqrtltpbts*costh5*(-1d0/8+1d0/8/s*spr)
     & +costh4*(-1d0/8*spr+1d0/8*s-1d0/8*mbt**2/s*spr+1d0/8*mbt**2+1d0/8
     & *mtp**2/s*spr-1d0/8*mtp**2)
     & +1d0/8*spr+1d0/8*s+1d0/8*mbt**2/s*spr+1d0/8*mbt**2-1d0/8*mtp**2/s
     & *spr-1d0/8*mtp**2

      p1p4=
     & +sqrtltpbts*sinph4*sinth4*sinth5*(-1d0/4*sqspr/sq)
     & +sqrtltpbts*costh4*costh5*(1d0/8+1d0/8/s*spr)
     & +sqrtltpbts*costh5*(-1d0/8+1d0/8/s*spr)
     & +costh4*(1d0/8*spr-1d0/8*s+1d0/8*mbt**2/s*spr-1d0/8*mbt**2-1d0/8*
     & mtp**2/s*spr+1d0/8*mtp**2)
     & +1d0/8*spr+1d0/8*s+1d0/8*mbt**2/s*spr+1d0/8*mbt**2-1d0/8*mtp**2/s
     & *spr-1d0/8*mtp**2

      p1p5=
     & +sqrtltpbts*costh5*(1d0/4-1d0/4/s*spr)
     & -1d0/4*spr+1d0/4*s-1d0/4*mbt**2/s*spr+1d0/4*mbt**2+1d0/4*mtp**2/s
     & *spr-1d0/4*mtp**2

      p2p3=
     & +sqrtltpbts*sinph4*sinth4*sinth5*(1d0/4*sqspr/sq)
     & +sqrtltpbts*costh4*costh5*(-1d0/8-1d0/8/s*spr)
     & +sqrtltpbts*costh5*(-1d0/8+1d0/8/s*spr)
     & +costh4*(1d0/8*spr-1d0/8*s-1d0/8*mbt**2/s*spr+1d0/8*mbt**2+1d0/8*
     & mtp**2/s*spr-1d0/8*mtp**2)
     & -1d0/8*spr-1d0/8*s+1d0/8*mbt**2/s*spr+1d0/8*mbt**2-1d0/8*mtp**2/s
     & *spr-1d0/8*mtp**2

      p2p4=
     & +sqrtltpbts*sinph4*sinth4*sinth5*(-1d0/4*sqspr/sq)
     & +sqrtltpbts*costh4*costh5*(1d0/8+1d0/8/s*spr)
     & +sqrtltpbts*costh5*(-1d0/8+1d0/8/s*spr)
     & +costh4*(-1d0/8*spr+1d0/8*s+1d0/8*mbt**2/s*spr-1d0/8*mbt**2-1d0/8
     & *mtp**2/s*spr+1d0/8*mtp**2)
     & -1d0/8*spr-1d0/8*s+1d0/8*mbt**2/s*spr+1d0/8*mbt**2-1d0/8*mtp**2/s
     & *spr-1d0/8*mtp**2

      p2p5=
     & +sqrtltpbts*costh5*(1d0/4-1d0/4/s*spr)
     & +1d0/4*spr-1d0/4*s-1d0/4*mbt**2/s*spr+1d0/4*mbt**2+1d0/4*mtp**2/s
     & *spr-1d0/4*mtp**2

      p3p4=
     & -1d0/2*spr

      p3p5=
     & +costh4*(1d0/4*spr-1d0/4*s)
     & +1d0/4*spr-1d0/4*s

      p4p5=
     & +costh4*(-1d0/4*spr+1d0/4*s)
     & +1d0/4*spr-1d0/4*s

      iz1bt=
     & +1d0/(mbt**2/s+1d0/4*sqrtltpbts**2*sinth5**2/s**2)/(s-spr)*(-1d0/
     & 2-1d0/2*mbt**2/s+1d0/2*mtp**2/s+1d0/2*sqrtltpbts*costh5/s)

      iz2tp=
     & +1d0/(mtp**2/s+1d0/4*sqrtltpbts**2*sinth5**2/s**2)/(s-spr)*(1d0/2
     & -1d0/2*mbt**2/s+1d0/2*mtp**2/s+1d0/2*sqrtltpbts*costh5/s)

      iz3up=
     & +1d0/(mup**2/spr+1d0/4*sqrtlsprupdn**2*sinth4**2/spr**2)/(s-spr)*
     & (1d0/2-1d0/2*mdn**2/spr+1d0/2*mup**2/spr-1d0/2*sqrtlsprupdn*costh
     & 4/spr)

      iz4dn=
     & +1d0/(mdn**2/spr+1d0/4*sqrtlsprupdn**2*sinth4**2/spr**2)/(s-spr)*
     & (1d0/2+1d0/2*mdn**2/spr-1d0/2*mup**2/spr+1d0/2*sqrtlsprupdn*costh
     & 4/spr)

      identp25m = (-2d0*p2p5-i_*mtp*wtp)
      dentp25m = 1d0/identp25m
      dentp25mc = dconjg(dentp25m)

      hardisr=
     & +iz1bt**2*theta_hard(s,spr,omega)*sqrtltpbts*propwspr*propwsprc/p
     & i**5*alpha*gf**2*qbt**2*mw**4/mtp**3*mbt**2/s*(s-spr)*(-3d0/32*p1
     & p3*p2p4-3d0/32*p2p4*p3p5)
     & +iz1bt*sqrtltpbts*dentp25m*propwspr*propwsprc/pi**5*alpha*gf**2*q
     & tp*qbt*mw**4/mtp**3/s*(s-spr)*(-3d0/32*p1p2*p1p3*p2p4+3d0/64*p1p2
     & *p1p3*p4p5-3d0/64*p1p2*p2p4*p3p5)
     & +iz1bt*sqrtltpbts*dentp25m*propwspr*propwsprc/pi**5*alpha*gf**2*q
     & tp*qbt*mw**4/mtp**3/s*(s-spr)**2*(3d0/128*p1p3*p1p4+3d0/128*p1p3*
     & p2p4)
     & +iz1bt*sqrtltpbts*dentp25mc*propwspr*propwsprc/pi**5*alpha*gf**2*
     & qtp*qbt*mw**4/mtp**3/s*(s-spr)*(-3d0/32*p1p2*p1p3*p2p4+3d0/64*p1p
     & 2*p1p3*p4p5-3d0/64*p1p2*p2p4*p3p5)
     & +iz1bt*sqrtltpbts*dentp25mc*propwspr*propwsprc/pi**5*alpha*gf**2*
     & qtp*qbt*mw**4/mtp**3/s*(s-spr)**2*(3d0/128*p1p3*p1p4+3d0/128*p1p3
     & *p2p4)
     & +iz1bt*sqrtltpbts*propwspr*propwsprc/pi**5*alpha*gf**2*qbt**2*mw*
     & *4/mtp**3/s*(s-spr)*(3d0/64*p2p4*p3p5)
     & +sqrtltpbts*dentp25m*dentp25mc*propwspr*propwsprc/pi**5*alpha*gf*
     & *2*qtp**2*mw**4/mtp**3/s*(s-spr)*(-3d0/32*p1p3*p2p5*p4p5)
     & +sqrtltpbts*dentp25m*dentp25mc*propwspr*propwsprc/pi**5*alpha*gf*
     & *2*qtp**2*mw**4/mtp/s*(s-spr)*(-3d0/32*p1p3*p2p4+3d0/32*p1p3*p4p5
     & )
     & +sqrtltpbts*dentp25m*propwspr*propwsprc/pi**5*alpha*gf**2*qtp*qbt
     & *mw**4/mtp**3/s*(s-spr)*(3d0/128*p1p3*p1p4-3d0/128*p2p3*p2p4)
     & +sqrtltpbts*dentp25mc*propwspr*propwsprc/pi**5*alpha*gf**2*qtp*qb
     & t*mw**4/mtp**3/s*(s-spr)*(3d0/128*p1p3*p1p4-3d0/128*p2p3*p2p4)

      hardfsr=
     & +iz3up**2*theta_hard(s,spr,omega)*sqrtltpbts*propws*propwsc/pi**5
     & *alpha*gf**2*qup**2*mw**4*mup**2/mtp**3/s*(s-spr)*(-3d0/32*p1p3*p
     & 2p4-3d0/32*p1p5*p2p4)
     & +iz3up*iz4dn*theta_hard(s,spr,omega)*sqrtltpbts*propws*propwsc/pi
     & **5*alpha*gf**2*qup*qdn*mw**4/mtp**3/s*(s-spr)*(-3d0/16*p1p3*p2p4
     & *p3p4-3d0/32*p1p3*p2p5*p3p4-3d0/32*p1p5*p2p4*p3p4)
     & +iz3up*sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qup*qdn*mw**4/
     & mtp**3/s*(s-spr)*(-3d0/64*p1p3*p2p3+3d0/64*p1p3*p2p4)
     & +iz3up*sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qup**2*mw**4/m
     & tp**3/s*(s-spr)*(3d0/64*p1p5*p2p4)
     & +iz4dn**2*theta_hard(s,spr,omega)*sqrtltpbts*propws*propwsc/pi**5
     & *alpha*gf**2*qdn**2*mw**4*mdn**2/mtp**3/s*(s-spr)*(-3d0/32*p1p3*p
     & 2p4-3d0/32*p1p3*p2p5)
     & +iz4dn*sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qdn**2*mw**4/m
     & tp**3/s*(s-spr)*(3d0/64*p1p3*p2p5)
     & +iz4dn*sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qup*qdn*mw**4/
     & mtp**3/s*(s-spr)*(3d0/64*p1p3*p2p4-3d0/64*p1p4*p2p4)

      hardifi=
     & +iz1bt*iz3up*theta_hard(s,spr,omega)*sqrtltpbts*propwspr*propwsc/
     & pi**5*alpha*gf**2*qup*qbt*mw**4/mtp**3/s*(s-spr)*(3d0/32*p1p3**2*
     & p2p4)
     & +iz1bt*iz3up*theta_hard(s,spr,omega)*sqrtltpbts*propws*propwsprc/
     & pi**5*alpha*gf**2*qup*qbt*mw**4/mtp**3/s*(s-spr)*(3d0/32*p1p3**2*
     & p2p4)
     & +iz1bt*iz4dn*theta_hard(s,spr,omega)*sqrtltpbts*propwspr*propwsc/
     & pi**5*alpha*gf**2*qdn*qbt*mw**4/mtp**3/s*(s-spr)*(-3d0/32*p1p3*p1
     & p4*p2p4)
     & +iz1bt*iz4dn*theta_hard(s,spr,omega)*sqrtltpbts*propwspr*propwsc/
     & pi**5*alpha*gf**2*qdn*qbt*mw**4/mtp**3/s*(s-spr)**2*(3d0/128*p1p3
     & *p1p4+3d0/128*p1p4*p2p4)
     & +iz1bt*iz4dn*theta_hard(s,spr,omega)*sqrtltpbts*propws*propwsprc/
     & pi**5*alpha*gf**2*qdn*qbt*mw**4/mtp**3/s*(s-spr)*(-3d0/32*p1p3*p1
     & p4*p2p4)
     & +iz1bt*iz4dn*theta_hard(s,spr,omega)*sqrtltpbts*propws*propwsprc/
     & pi**5*alpha*gf**2*qdn*qbt*mw**4/mtp**3/s*(s-spr)**2*(3d0/128*p1p3
     & *p1p4+3d0/128*p1p4*p2p4)
     & +iz1bt*sqrtltpbts*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qbt*mw*
     & *4/mtp**3/s*(3d0/16*p1p2*p1p3*p2p4-3d0/32*p1p2*p1p3*p4p5+3d0/32*p
     & 1p2*p2p4*p3p5)
     & +iz1bt*sqrtltpbts*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qbt*mw*
     & *4/mtp**3/s*(s-spr)*(-3d0/64*p1p3*p1p4-3d0/64*p1p3*p2p4)
     & +iz1bt*sqrtltpbts*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qbt*mw*
     & *4/mtp**3*mbt**2/s*(3d0/16*p1p3*p2p4+3d0/16*p2p4*p3p5)
     & +iz1bt*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qdn*qbt*mw**
     & 4/mtp**3/s*(s-spr)*(-3d0/128*p1p2*p1p3+3d0/128*p1p3*p2p4-3d0/128*
     & p1p4*p2p4)
     & +iz1bt*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qup*qbt*mw**
     & 4/mtp**3/s*(s-spr)*(-3d0/64*p1p3*p2p4)
     & +iz1bt*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qup*qbt*mw**
     & 4/mtp**3*mbt**2/s*(s-spr)*(-3d0/128*p2p4)
     & +iz1bt*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qbt*mw**4
     & /mtp**3/s*(-3d0/32*p1p2*p1p3*p2p4+3d0/64*p1p2*p1p3*p4p5-3d0/64*p1
     & p2*p2p4*p3p5)
     & +iz1bt*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qbt*mw**4
     & /mtp**3/s*(s-spr)*(3d0/128*p1p3*p1p4+3d0/128*p1p3*p2p4)
     & +iz1bt*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qbt*mw**4
     & /mtp**3*mbt**2/s*(-3d0/32*p1p3*p2p4-3d0/32*p2p4*p3p5)
     & +iz1bt*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qdn*qbt*mw**
     & 4/mtp**3/s*(s-spr)*(-3d0/128*p1p2*p1p3+3d0/128*p1p3*p2p4-3d0/128*
     & p1p4*p2p4)
     & +iz1bt*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qup*qbt*mw**
     & 4/mtp**3/s*(s-spr)*(-3d0/64*p1p3*p2p4)
     & +iz1bt*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qup*qbt*mw**
     & 4/mtp**3*mbt**2/s*(s-spr)*(-3d0/128*p2p4)
     & +iz1bt*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qbt*mw**4
     & /mtp**3/s*(-3d0/32*p1p2*p1p3*p2p4+3d0/64*p1p2*p1p3*p4p5-3d0/64*p1
     & p2*p2p4*p3p5)
     & +iz1bt*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qbt*mw**4
     & /mtp**3/s*(s-spr)*(3d0/128*p1p3*p1p4+3d0/128*p1p3*p2p4)
     & +iz1bt*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qbt*mw**4
     & /mtp**3*mbt**2/s*(-3d0/32*p1p3*p2p4-3d0/32*p2p4*p3p5)
     & +iz3up*sqrtltpbts*dentp25m*propwspr*propwsc/pi**5*alpha*gf**2*qup
     & *qtp*mw**4/mtp**3/s*(s-spr)*(-3d0/32*p1p3*p2p3*p2p4-3d0/64*p1p3*p
     & 2p4*p2p5-3d0/64*p1p3*p2p5*p3p4-3d0/64*p1p5*p2p3*p2p4)
     & +iz3up*sqrtltpbts*dentp25m*propwspr*propwsc/pi**5*alpha*gf**2*qup
     & *qtp*mw**4/mtp**3/s*(s-spr)**2*(-3d0/128*p1p3*p2p3)
     & +iz3up*sqrtltpbts*dentp25mc*propws*propwsprc/pi**5*alpha*gf**2*qu
     & p*qtp*mw**4/mtp**3/s*(s-spr)*(-3d0/32*p1p3*p2p3*p2p4-3d0/64*p1p3*
     & p2p4*p2p5-3d0/64*p1p3*p2p5*p3p4-3d0/64*p1p5*p2p3*p2p4)
     & +iz3up*sqrtltpbts*dentp25mc*propws*propwsprc/pi**5*alpha*gf**2*qu
     & p*qtp*mw**4/mtp**3/s*(s-spr)**2*(-3d0/128*p1p3*p2p3)
     & +iz3up*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qup*qbt*mw**
     & 4/mtp**3/s*(s-spr)*(-3d0/64*p1p3*p2p4)
     & +iz3up*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qup*mw**4
     & /mtp**3/s*(3d0/32*p1p3*p2p4*p3p4+3d0/64*p1p3*p2p5*p3p4+3d0/64*p1p
     & 5*p2p4*p3p4)
     & +iz3up*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qup*mw**4
     & /mtp**3/s*(s-spr)*(3d0/128*p1p3*p2p3-3d0/128*p1p3*p2p4)
     & +iz3up*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qup*qbt*mw**
     & 4/mtp**3/s*(s-spr)*(-3d0/64*p1p3*p2p4)
     & +iz3up*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qup*mw**4
     & /mtp**3/s*(3d0/32*p1p3*p2p4*p3p4+3d0/64*p1p3*p2p5*p3p4+3d0/64*p1p
     & 5*p2p4*p3p4)
     & +iz3up*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qup*mw**4
     & /mtp**3/s*(s-spr)*(3d0/128*p1p3*p2p3-3d0/128*p1p3*p2p4)
     & +iz3up*sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw*qup*mw**4/m
     & tp**3/s*(-3d0/16*p1p3*p2p4*p3p4-3d0/32*p1p3*p2p5*p3p4-3d0/32*p1p5
     & *p2p4*p3p4)
     & +iz3up*sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw*qup*mw**4/m
     & tp**3/s*(s-spr)*(-3d0/64*p1p3*p2p3+3d0/64*p1p3*p2p4)
     & +iz4dn*sqrtltpbts*dentp25m*propwspr*propwsc/pi**5*alpha*gf**2*qdn
     & *qtp*mw**4/mtp**3/s*(s-spr)*(3d0/32*p1p3*p2p4*p2p5+3d0/32*p1p3*p2
     & p4**2)
     & +iz4dn*sqrtltpbts*dentp25mc*propws*propwsprc/pi**5*alpha*gf**2*qd
     & n*qtp*mw**4/mtp**3/s*(s-spr)*(3d0/32*p1p3*p2p4*p2p5+3d0/32*p1p3*p
     & 2p4**2)
     & +iz4dn*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qdn*qbt*mw**
     & 4/mtp**3/s*(s-spr)*(3d0/128*p1p3*p1p4+3d0/128*p1p3*p2p4-3d0/128*p
     & 2p4*p3p4)
     & +iz4dn*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qdn*mw**4
     & /mtp**3/s*(-3d0/32*p1p3*p2p4*p3p4-3d0/64*p1p3*p2p5*p3p4-3d0/64*p1
     & p5*p2p4*p3p4)
     & +iz4dn*sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qdn*mw**4
     & /mtp**3/s*(s-spr)*(3d0/128*p1p3*p2p4-3d0/128*p1p4*p2p4)
     & +iz4dn*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qdn*qbt*mw**
     & 4/mtp**3/s*(s-spr)*(3d0/128*p1p3*p1p4+3d0/128*p1p3*p2p4-3d0/128*p
     & 2p4*p3p4)
     & +iz4dn*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qdn*mw**4
     & /mtp**3/s*(-3d0/32*p1p3*p2p4*p3p4-3d0/64*p1p3*p2p5*p3p4-3d0/64*p1
     & p5*p2p4*p3p4)
     & +iz4dn*sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qdn*mw**4
     & /mtp**3/s*(s-spr)*(3d0/128*p1p3*p2p4-3d0/128*p1p4*p2p4)
     & +iz4dn*sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw*qdn*mw**4/m
     & tp**3/s*(3d0/16*p1p3*p2p4*p3p4+3d0/32*p1p3*p2p5*p3p4+3d0/32*p1p5*
     & p2p4*p3p4)
     & +iz4dn*sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw*qdn*mw**4/m
     & tp**3/s*(s-spr)*(-3d0/64*p1p3*p2p4+3d0/64*p1p4*p2p4)
     & +sqrtltpbts*dentp25m*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qtp*
     & mw**4/mtp**3/s*(3d0/32*p1p2*p1p3*p2p4-3d0/64*p1p2*p1p3*p4p5+3d0/6
     & 4*p1p2*p2p4*p3p5-3d0/64*p1p3*p2p4*p2p5-3d0/64*p1p3*p2p5*p3p4+3d0/
     & 64*p1p3*p2p5*p4p5-3d0/64*p1p5*p2p3*p2p4)
     & +sqrtltpbts*dentp25m*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qtp*
     & mw**4/mtp**3/s*(s-spr)*(-3d0/128*p1p3*p2p4)
     & +sqrtltpbts*dentp25m*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qtp*
     & mw**4/mtp/s*(3d0/32*p1p3*p2p4-3d0/32*p1p3*p4p5)
     & +sqrtltpbts*dentp25m*propwspr*propwsc/pi**5*alpha*gf**2*qdn*qtp*m
     & w**4/mtp**3/s*(s-spr)*(3d0/64*p1p3*p2p4+3d0/64*p1p3*p2p5)
     & +sqrtltpbts*dentp25m*propwspr*propwsc/pi**5*alpha*gf**2*qdn*qtp*m
     & w**4/mtp/s*(s-spr)*(-3d0/128*p1p3)
     & +sqrtltpbts*dentp25m*propwspr*propwsc/pi**5*alpha*gf**2*qup*qtp*m
     & w**4/mtp**3/s*(s-spr)*(-3d0/128*p1p2*p2p4+3d0/128*p1p3*p2p3-3d0/1
     & 28*p1p3*p2p4)
     & +sqrtltpbts*dentp25m*propwspr*propwsc/pi**5*alpha*gf**2*qw*qtp*mw
     & **4/mtp**3/s*(-3d0/32*p1p2*p1p3*p2p4+3d0/64*p1p2*p1p3*p4p5-3d0/64
     & *p1p2*p2p4*p3p5+3d0/64*p1p3*p2p4*p2p5+3d0/64*p1p3*p2p5*p3p4-3d0/6
     & 4*p1p3*p2p5*p4p5+3d0/64*p1p5*p2p3*p2p4)
     & +sqrtltpbts*dentp25m*propwspr*propwsc/pi**5*alpha*gf**2*qw*qtp*mw
     & **4/mtp**3/s*(s-spr)*(3d0/128*p1p3*p2p4)
     & +sqrtltpbts*dentp25m*propwspr*propwsc/pi**5*alpha*gf**2*qw*qtp*mw
     & **4/mtp/s*(-3d0/32*p1p3*p2p4+3d0/32*p1p3*p4p5)
     & +sqrtltpbts*dentp25mc*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qtp
     & *mw**4/mtp**3/s*(3d0/32*p1p2*p1p3*p2p4-3d0/64*p1p2*p1p3*p4p5+3d0/
     & 64*p1p2*p2p4*p3p5-3d0/64*p1p3*p2p4*p2p5-3d0/64*p1p3*p2p5*p3p4+3d0
     & /64*p1p3*p2p5*p4p5-3d0/64*p1p5*p2p3*p2p4)
     & +sqrtltpbts*dentp25mc*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qtp
     & *mw**4/mtp**3/s*(s-spr)*(-3d0/128*p1p3*p2p4)
     & +sqrtltpbts*dentp25mc*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qtp
     & *mw**4/mtp/s*(3d0/32*p1p3*p2p4-3d0/32*p1p3*p4p5)
     & +sqrtltpbts*dentp25mc*propws*propwsprc/pi**5*alpha*gf**2*qdn*qtp*
     & mw**4/mtp**3/s*(s-spr)*(3d0/64*p1p3*p2p4+3d0/64*p1p3*p2p5)
     & +sqrtltpbts*dentp25mc*propws*propwsprc/pi**5*alpha*gf**2*qdn*qtp*
     & mw**4/mtp/s*(s-spr)*(-3d0/128*p1p3)
     & +sqrtltpbts*dentp25mc*propws*propwsprc/pi**5*alpha*gf**2*qup*qtp*
     & mw**4/mtp**3/s*(s-spr)*(-3d0/128*p1p2*p2p4+3d0/128*p1p3*p2p3-3d0/
     & 128*p1p3*p2p4)
     & +sqrtltpbts*dentp25mc*propws*propwsprc/pi**5*alpha*gf**2*qw*qtp*m
     & w**4/mtp**3/s*(-3d0/32*p1p2*p1p3*p2p4+3d0/64*p1p2*p1p3*p4p5-3d0/6
     & 4*p1p2*p2p4*p3p5+3d0/64*p1p3*p2p4*p2p5+3d0/64*p1p3*p2p5*p3p4-3d0/
     & 64*p1p3*p2p5*p4p5+3d0/64*p1p5*p2p3*p2p4)
     & +sqrtltpbts*dentp25mc*propws*propwsprc/pi**5*alpha*gf**2*qw*qtp*m
     & w**4/mtp**3/s*(s-spr)*(3d0/128*p1p3*p2p4)
     & +sqrtltpbts*dentp25mc*propws*propwsprc/pi**5*alpha*gf**2*qw*qtp*m
     & w**4/mtp/s*(-3d0/32*p1p3*p2p4+3d0/32*p1p3*p4p5)
     & +sqrtltpbts*propwspr*propwsprc/pi**5*alpha*gf**2*qw*qbt*mw**4/mtp
     & **3/s*(-3d0/64*p1p3*p1p4-3d0/64*p1p3*p2p4+3d0/64*p2p4*p3p4-3d0/64
     & *p2p4*p3p5)
     & +sqrtltpbts*propwspr*propwsprc/pi**5*alpha*gf**2*qw**2*mw**4/mtp*
     & *3/s/(s-spr)*(3d0/32*p1p2*p1p3*p4p5-3d0/32*p1p2*p2p4*p3p5+3d0/32*
     & p1p3*p2p5*p3p4+3d0/32*p1p5*p2p4*p3p4)
     & +sqrtltpbts*propwspr*propwsprc/pi**5*alpha*gf**2*qw**2*mw**4/mtp*
     & *3/(s-spr)*(-3d0/32*p1p3*p2p4)
     & +sqrtltpbts*propwspr*propwsprc/pi**5*alpha*gf**2*qw**2*mw**4/mtp*
     & *3*mbt**2/s/(s-spr)*(-3d0/32*p2p4*p3p5)
     & +sqrtltpbts*propwspr*propwsprc/pi**5*alpha*gf**2*qw**2*mw**4/mtp/
     & s/(s-spr)*(3d0/32*p1p3*p4p5)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qup*qbt*mw**4/mtp*
     & *3/s*(s-spr)*(3d0/128*p2p4)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qbt*mw**4/mtp**
     & 3/s*(3d0/128*p1p3*p1p4+3d0/128*p1p3*p2p4-3d0/128*p2p4*p3p4+3d0/12
     & 8*p2p4*p3p5)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qdn*mw**4/mtp**
     & 3/s*(3d0/128*p1p2*p1p3+3d0/128*p1p3*p2p4-3d0/128*p1p3*p2p5+3d0/12
     & 8*p1p4*p2p4)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qdn*mw**4/mtp/s
     & *(3d0/128*p1p3)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qup*mw**4/mtp**
     & 3/s*(3d0/128*p1p2*p2p4-3d0/128*p1p3*p2p3-3d0/128*p1p3*p2p4+3d0/12
     & 8*p1p5*p2p4)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw*qup*mw**4/mtp**
     & 3*mbt**2/s*(3d0/128*p2p4)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw**2*mw**4/mtp**3
     & /s/(s-spr)*(-3d0/32*p1p2*p1p3*p4p5+3d0/32*p1p2*p2p4*p3p5-3d0/32*p
     & 1p3*p2p5*p3p4-3d0/32*p1p5*p2p4*p3p4)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw**2*mw**4/mtp**3
     & /(s-spr)*(3d0/32*p1p3*p2p4)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw**2*mw**4/mtp**3
     & *mbt**2/s/(s-spr)*(3d0/32*p2p4*p3p5)
     & +sqrtltpbts*propwspr*propwsc/pi**5*alpha*gf**2*qw**2*mw**4/mtp/s/
     & (s-spr)*(-3d0/32*p1p3*p4p5)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qup*qbt*mw**4/mtp*
     & *3/s*(s-spr)*(3d0/128*p2p4)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qbt*mw**4/mtp**
     & 3/s*(3d0/128*p1p3*p1p4+3d0/128*p1p3*p2p4-3d0/128*p2p4*p3p4+3d0/12
     & 8*p2p4*p3p5)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qdn*mw**4/mtp**
     & 3/s*(3d0/128*p1p2*p1p3+3d0/128*p1p3*p2p4-3d0/128*p1p3*p2p5+3d0/12
     & 8*p1p4*p2p4)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qdn*mw**4/mtp/s
     & *(3d0/128*p1p3)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qup*mw**4/mtp**
     & 3/s*(3d0/128*p1p2*p2p4-3d0/128*p1p3*p2p3-3d0/128*p1p3*p2p4+3d0/12
     & 8*p1p5*p2p4)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw*qup*mw**4/mtp**
     & 3*mbt**2/s*(3d0/128*p2p4)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw**2*mw**4/mtp**3
     & /s/(s-spr)*(-3d0/32*p1p2*p1p3*p4p5+3d0/32*p1p2*p2p4*p3p5-3d0/32*p
     & 1p3*p2p5*p3p4-3d0/32*p1p5*p2p4*p3p4)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw**2*mw**4/mtp**3
     & /(s-spr)*(3d0/32*p1p3*p2p4)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw**2*mw**4/mtp**3
     & *mbt**2/s/(s-spr)*(3d0/32*p2p4*p3p5)
     & +sqrtltpbts*propws*propwsprc/pi**5*alpha*gf**2*qw**2*mw**4/mtp/s/
     & (s-spr)*(-3d0/32*p1p3*p4p5)
     & +sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw*qdn*mw**4/mtp**3/
     & s*(-3d0/64*p1p2*p1p3-3d0/64*p1p3*p2p4+3d0/64*p1p3*p2p5-3d0/64*p1p
     & 4*p2p4)
     & +sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw*qdn*mw**4/mtp/s*(
     & -3d0/64*p1p3)
     & +sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw*qup*mw**4/mtp**3/
     & s*(-3d0/64*p1p2*p2p4+3d0/64*p1p3*p2p3+3d0/64*p1p3*p2p4-3d0/64*p1p
     & 5*p2p4)
     & +sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw*qup*mw**4/mtp**3*
     & mbt**2/s*(-3d0/64*p2p4)
     & +sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw**2*mw**4/mtp**3/s
     & /(s-spr)*(3d0/32*p1p2*p1p3*p4p5-3d0/32*p1p2*p2p4*p3p5+3d0/32*p1p3
     & *p2p5*p3p4+3d0/32*p1p5*p2p4*p3p4)
     & +sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw**2*mw**4/mtp**3/(
     & s-spr)*(-3d0/32*p1p3*p2p4)
     & +sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw**2*mw**4/mtp**3*m
     & bt**2/s/(s-spr)*(-3d0/32*p2p4*p3p5)
     & +sqrtltpbts*propws*propwsc/pi**5*alpha*gf**2*qw**2*mw**4/mtp/s/(s
     & -spr)*(3d0/32*p1p3*p4p5)

      if (iqed.eq.1) then
         hard = hardisr+hardifi+hardfsr
      elseif(iqed.eq.2) then
         hard = hardisr
      elseif(iqed.eq.3) then
         hard = hardifi
      elseif(iqed.eq.4) then
         hard = hardfsr
      elseif(iqed.eq.5) then
         hard = hardifi+hardfsr
      endif

      hard = hard*cfprime

      return
      end
