************************************************************************
* sanc_cc_v1.40 package.
************************************************************************
* File (process_cc_1loop_reaction.F).
************************************************************************
* This is the FORTRAN subroutine module to calculate 1-loop EW and QCD
* radiative correction for the Charged Current Drell-Yan processes
* (reaction).
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2011.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine Process_Reaction
     &(born,born_sd,virt,virt_sd,viso,viso_sd,brdq,brdq_sd,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 ihard,ome,pid
      integer*4 ncall1,itmx1,itmx2
      integer*4 nrdm,nc1(10),it1(10),it2(10)
      real*8 sq,Mfscale,vud,vtb
      real*8 born_sd,virt_sd,viso,viso_sd,brdq,brdq_sd,hard_sd
      real*8 virt_ew,virt_ew_sd,virt_qcd,virt_qcd_sd,viso_ew,
     &     viso_qcd,brdq_ew,brdq_qcd,hard_ew,hard_qcd,born1
      real*8 viso_ew_sd,viso_qcd_sd,brdq_ew_sd,brdq_qcd_sd,hard_ew_sd,
     &     hard_qcd_sd

      common /s/ s,sq
      common /Mfscale/ Mfscale
      common /ihard/ ihard
      common /pid/ pid
      common /vegas_param/ nrdm,nc1,it1,it2

      born        = 0d0
      born_sd     = 0d0
      virt        = 0d0
      virt_sd     = 0d0
      virt_ew     = 0d0
      virt_ew_sd  = 0d0
      virt_qcd    = 0d0
      virt_qcd_sd = 0d0
      viso_ew     = 0d0
      viso_ew_sd  = 0d0
      viso_qcd    = 0d0
      viso_qcd_sd = 0d0
      brdq        = 0d0
      brdq_sd     = 0d0
      brdq_ew     = 0d0
      brdq_ew_sd  = 0d0
      brdq_qcd    = 0d0
      brdq_qcd_sd = 0d0
      hard        = 0d0
      hard_sd     = 0d0
      hard_ew     = 0d0
      hard_ew_sd  = 0d0
      hard_qcd    = 0d0
      hard_qcd_sd = 0d0

      if (ihard.eq.1.and.iqed.eq.1.and.(pid.eq.25.or.pid.eq.26)) then
         print *, "wrong value for ihard", ihard
         stop
      endif

*     born
      iborn  = 1
      ncall1 = nc1(1)
      itmx1  = it1(1)
      itmx2  = it2(1)
      call ProcessBorn_R
     &     (nrdm,ncall1,itmx1,itmx2,born,born_sd)

      print *, "sq(GeV)        = ", sq
      print *, "[bo](pb)       = ", born, born_sd
      if (iqed.eq.0.and.iew.eq.0.and.iqcd.eq.0) stop

      print *, ""
      print *, "omega          = ", omega

*     EW part
      if (iew.ne.0) then
*     virt (Weak)
         iborn = 0
         ncall1 = nc1(2) 
         itmx1  = it1(2)
         itmx2  = it2(2)
         call ProcessVirt_R_EW
     &        (nrdm,ncall1,itmx1,itmx2,virt_ew,virt_ew_sd)
      endif

      if (iqed.ne.0) then
*     virt+soft (QED)
         iborn = 0
         ncall1 = nc1(3)
         itmx1  = it1(3)
         itmx2  = it2(3)
         call ProcessViSo_R_EW
     &        (nrdm,ncall1,itmx1,itmx2,viso_ew,viso_ew_sd)
*     brdq (QED)
         iborn = 0
         ncall1 = nc1(4)
         itmx1  = it1(4)
         itmx2  = it2(4)
         call ProcessBrdq_R_EW
     &        (nrdm,ncall1,itmx1,itmx2,brdq_ew,brdq_ew_sd)

         if (ihard.eq.1) then
*     hard (QED), 1spr
            iborn = 0
            ncall1 = nc1(5)
            itmx1  = it1(5)
            itmx2  = it2(5)
            call ProcessHard_R_EW_1spr
     &           (nrdm,ncall1,itmx1,itmx2,hard_ew,hard_ew_sd)

         elseif (ihard.eq.2) then
*     hard (QED), 4d
            iborn = 0
            ncall1 = nc1(6)
            itmx1  = it1(6)
            itmx2  = it2(6)
            call ProcessHard_R_EW
     &           (nrdm,ncall1,itmx1,itmx2,hard_ew,hard_ew_sd)
         endif
      endif

*     QCD part
      if (iqcd.ne.0) then
*     virt+soft (QCD)
         iborn = 0
         ncall1 = nc1(7)
         itmx1  = it1(7)
         itmx2  = it2(7)
         call ProcessViSo_R_QCD
     &        (nrdm,ncall1,itmx1,itmx2,viso_qcd,viso_qcd_sd)
*     brdq (QCD)
         iborn = 0
         ncall1 = nc1(8)
         itmx1  = it1(8)
         itmx2  = it2(8)
            call ProcessBrdq_R_QCD
     &           (nrdm,ncall1,itmx1,itmx2,brdq_qcd,brdq_qcd_sd)

         if (ihard.eq.1) then
*     hard (QCD), 1spr
            iborn = 0
            ncall1 = nc1(9)
            itmx1  = it1(9)
            itmx2  = it2(9)
               call ProcessHard_R_QCD_1spr
     &              (nrdm,ncall1,itmx1,itmx2,hard_qcd,hard_qcd_sd)

         elseif (ihard.eq.2) then
*     hard, 4d
            iborn = 0
            ncall1 = nc1(10)
            itmx1  = it1(10)
            itmx2  = it2(10)
               call ProcessHard_R_QCD
     &              (nrdm,ncall1,itmx1,itmx2,hard_qcd,hard_qcd_sd)
         endif
      endif

      virt    = virt_ew !- born
      virt_sd = virt_ew_sd
      viso    = viso_ew + viso_qcd
      viso_sd = dsqrt(viso_ew_sd**2 + viso_qcd_sd**2)
      brdq    = brdq_ew + brdq_qcd
      brdq_sd = dsqrt(brdq_ew_sd**2 + brdq_qcd_sd**2)
      hard    = hard_ew + hard_qcd
      hard_sd = dsqrt(hard_ew_sd**2 + hard_qcd_sd**2)

      return
      end
************************************************************************
* Born, reaction, EW
************************************************************************
      subroutine ProcessBorn_R
     &     (nrdm,ncall1,itmx1,itmx2,born,born_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,nrdm
      real*8 born_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      external function_born_r_ew_1c

      xl(1) = -1d0
      xu(1) = +1d0

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_born_r_ew_1c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_born_r_ew_1c, avgi, sd, chi2a)
      born = avgi
      born_sd = sd

      return
      end
************************************************************************
      real*8 function function_born_r_ew_1c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 sq,x(10),wgt,born1,sqrtlsdntp,sqrtlsbtup,sqrlstpbt
      common /s/ s,sq
      common /pid/ pid

      cosf = x(1)

      if     ((pid.eq.1).or.(pid.eq.2)) then
         t = - s/2*(1+cosf)
         u = - s/2*(1-cosf)
      elseif ((pid.eq.3).or.(pid.eq.4).or.(pid.eq.5).or.(pid.eq.6)) then
         t = - (s-mf**2)/2*(1+cosf)
         u = - (s-mf**2)/2*(1-cosf)
      elseif ((pid.eq.25).or.(pid.eq.26)) then
         t = mbt**2 + 1d0/2*(s+mbt**2-mtp**2)*(-1-betaf*cosf)
         u = mbt**2 + 1d0/2*(s+mbt**2-mtp**2)*(-1+betaf*cosf)
      elseif ((pid.eq.27).or.(pid.eq.28).or.(pid.eq.29).or.(pid.eq.30)) then
         t =
     &        + (s-mtp**2) * (  - 1d0/2 )
     &        + cosf*(s-mtp**2) * ( + 1d0/2/s*(s-mbt**2) )
     &        + 1d0/2*mbt**2 - 1d0/2*mtp**2*mbt**2/s
         u =
     &        + (s-mtp**2) * (  - 1d0/2 )
     &        + cosf*(s-mtp**2) * ( - 1d0/2/s*(s-mbt**2) )
     &        + 1d0/2*mbt**2 + 1d0/2*mtp**2*mbt**2/s
      endif

      function_born_r_ew_1c = 0d0
      if     (pid.eq.1) then
         call cc_br_1314_1112 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.2) then
         call cc_br_1413_1211 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.3) then
         call cc_br_1314_1516 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.4) then
         call cc_br_1413_1615 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.5) then
         call cc_br_1314_1920 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.6) then
         call cc_br_1413_2019 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.25) then
         call cc_br_1314_2122 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.26) then
         call cc_br_1413_2221 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.27) then
         call cc_br_2213_1421 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.28) then
         call cc_br_1322_2114 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.29) then
         call cc_br_1422_1321 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.30) then
         call cc_br_2214_2113 (-s,-t,-u,born,soft,hard)
      endif
      function_born_r_ew_1c = born

      return
      end
************************************************************************
* Virt (Weak), reaction, EW
************************************************************************
      subroutine ProcessVirt_R_EW(nrdm,ncall1,itmx1,itmx2,virt,virt_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,nrdm
      real*8 virt_sd,x1cut
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      external function_virt_r_ew_1c

      x1cut = 1d-13
      xl(1) = -1d0+x1cut
      xu(1) = +1d0-x1cut

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_virt_r_ew_1c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_virt_r_ew_1c, avgi, sd, chi2a)
      virt = avgi
      virt_sd = sd

      return
      end
************************************************************************
      real*8 function function_virt_r_ew_1c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 sq,mbt_
      real*8 x(10),wgt
      common /s/ s,sq
      common /pid/ pid

      cosf = x(1)
c      cosf = -0.7d0

      if     ((pid.eq.1).or.(pid.eq.2)) then
         t = - s/2*(1+cosf)
         u = - s/2*(1-cosf)
      elseif ((pid.eq.3).or.(pid.eq.4).or.(pid.eq.5).or.(pid.eq.6)) then
         t = - (s-mf**2)/2*(1+cosf)
         u = - (s-mf**2)/2*(1-cosf)
      elseif ((pid.eq.25).or.(pid.eq.26)) then
         t = -1d0/2*(s-mtp**2)*(1+cosf)
         u = -1d0/2*(s-mtp**2)*(1-cosf)
      elseif ((pid.eq.27).or.(pid.eq.28).or.(pid.eq.29).or.(pid.eq.30)) then
         t =
     &        + (s-mtp**2) * (  - 1d0/2 )
     &        + cosf*(s-mtp**2) * ( + 1d0/2 )
         u =
     &        + (s-mtp**2) * (  - 1d0/2 )
     &        + cosf*(s-mtp**2) * ( - 1d0/2 )
      endif

      function_virt_r_ew_1c = 0d0
      sigma = 0d0
      if     (pid.eq.1) then
         call cc_br_1314_1112 (-s,-t,-u,born,soft,hard)
         call cc_si_1314_1112 (s,t,u,sigma)
      elseif (pid.eq.2) then
         call cc_br_1413_1211 (-s,-t,-u,born,soft,hard)
         call cc_si_1413_1211 (s,t,u,sigma)
      elseif (pid.eq.3) then
         call cc_br_1314_1516 (-s,-t,-u,born,soft,hard)
         call cc_si_1314_1516 (s,t,u,sigma)
      elseif (pid.eq.4) then
         call cc_br_1413_1615 (-s,-t,-u,born,soft,hard)
         call cc_si_1413_1615 (s,t,u,sigma)
      elseif (pid.eq.5) then
         call cc_br_1314_1920 (-s,-t,-u,born,soft,hard)
         call cc_si_1314_1920 (s,t,u,sigma)
      elseif (pid.eq.6) then
         call cc_br_1413_2019 (-s,-t,-u,born,soft,hard)
         call cc_si_1413_2019 (s,t,u,sigma)
      elseif (pid.eq.25) then
         mbt_ = mbt
         mbt  = 0d0
         call cc_br_1314_2122 (-s,-t,-u,born,soft,hard)
         call cc_si_1314_2122 (s,t,u,sigma)
         mbt = mbt_
      elseif (pid.eq.26) then
         mbt_ = mbt
         mbt  = 0d0
         call cc_br_1413_2221 (-s,-t,-u,born,soft,hard)
         call cc_si_1413_2221 (s,t,u,sigma)
         mbt = mbt_
      elseif (pid.eq.27) then
         mbt_ = mbt
         mbt  = 0d0
         call cc_br_2213_1421 (-s,-t,-u,born,soft,hard)
         call cc_si_2213_1421 (s,t,u,sigma)
         mbt = mbt_
      elseif (pid.eq.28) then
         mbt_ = mbt
         mbt  = 0d0
         call cc_br_1322_2114 (-s,-t,-u,born,soft,hard)
         call cc_si_1322_2114 (s,t,u,sigma)
         mbt = mbt_
      elseif (pid.eq.29) then
         mbt_ = mbt
         mbt  = 0d0
         call cc_br_1422_1321 (-s,-t,-u,born,soft,hard)
         call cc_si_1422_1321 (s,t,u,sigma)
         mbt = mbt_
      elseif (pid.eq.30) then
         mbt_ = mbt
         mbt  = 0d0
         call cc_br_2214_2113 (-s,-t,-u,born,soft,hard)
         call cc_si_2214_2113 (s,t,u,sigma)
         mbt = mbt_
      endif
      function_virt_r_ew_1c = sigma - born

      return
      end
************************************************************************
* Virt+Soft (QED), reaction, EW
************************************************************************
      subroutine ProcessViSo_R_EW(nrdm,ncall1,itmx1,itmx2,viso,viso_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,nrdm
      real*8 viso,viso_sd,x1cut
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      external function_viso_r_ew_1c

      x1cut = 1d-15
      xl(1) = -1d0+x1cut
      xu(1) = +1d0-x1cut

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_viso_r_ew_1c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_viso_r_ew_1c, avgi, sd, chi2a)
      viso = avgi
      viso_sd = sd

      return
      end
************************************************************************
      real*8 function function_viso_r_ew_1c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 sq,sqrtlsdntp,sqrtlsbtup,sqrlstpbt
      real*8 x(10),wgt
      common /s/ s,sq
      common /pid/ pid

      cosf = x(1)

      if     ((pid.eq.1).or.(pid.eq.2)) then
         t = - s/2*(1+cosf)
         u = - s/2*(1-cosf)
      elseif ((pid.eq.3).or.(pid.eq.4).or.(pid.eq.5).or.(pid.eq.6)) then
         t = - (s-mf**2)/2*(1+cosf)
         u = - (s-mf**2)/2*(1-cosf)
      elseif ((pid.eq.25).or.(pid.eq.26)) then
         t = mbt**2 - 1d0/2*(s+mbt**2-mtp**2)*(1+betaf*cosf)
         u = mbt**2 - 1d0/2*(s+mbt**2-mtp**2)*(1-betaf*cosf)
      elseif ((pid.eq.27).or.(pid.eq.28).or.(pid.eq.29).or.(pid.eq.30)) then
         t =
     &        + (s-mtp**2) * (  - 1d0/2 )
     &        + cosf*(s-mtp**2) * ( + 1d0/2/s*(s-mbt**2) )
     &        + 1d0/2*mbt**2 - 1d0/2*mtp**2*mbt**2/s
         u =
     &        + (s-mtp**2) * (  - 1d0/2 )
     &        + cosf*(s-mtp**2) * ( - 1d0/2/s*(s-mbt**2) )
     &        + 1d0/2*mbt**2 + 1d0/2*mtp**2*mbt**2/s
      endif

      function_viso_r_ew_1c = 0d0
      if     (pid.eq.1) then
         call cc_br_1314_1112 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.2) then
         call cc_br_1413_1211 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.3) then
         call cc_br_1314_1516 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.4) then
         call cc_br_1413_1615 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.5) then
         call cc_br_1314_1920 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.6) then
         call cc_br_1413_2019 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.25) then
         call cc_br_1314_2122 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.26) then
         call cc_br_1413_2221 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.27) then
         call cc_br_2213_1421 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.28) then
         call cc_br_1322_2114 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.29) then
         call cc_br_1422_1321 (-s,-t,-u,born,soft,hard)
      elseif (pid.eq.30) then
         call cc_br_2214_2113 (-s,-t,-u,born,soft,hard)
      endif
      function_viso_r_ew_1c = soft

      return
      end
************************************************************************
* Brdq, reaction, EW
************************************************************************
      subroutine ProcessBrdq_R_EW(nrdm,ncall1,itmx1,itmx2,brdq,brdq_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 brdq,brdq_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      real*8 DelSV_MS,Mfscale,sq
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s,sq
      common /Mfscale/ Mfscale
      common /pid/ pid
      external function_brdq_r_ew_1s

      DelSV_MS = 0d0
      brdq     = 0d0
      brdq_sd  = 0d0

      if     (iqed.eq.0) return

      if     (pid.eq.1) then
         call cc_bo_1314_1112 (s,born)
      elseif (pid.eq.2) then
         call cc_bo_1413_1211 (s,born)
      elseif (pid.eq.3) then
         call cc_bo_1314_1516 (s,born)
      elseif (pid.eq.4) then
         call cc_bo_1413_1615 (s,born)
      elseif (pid.eq.5) then
         call cc_bo_1314_1920 (s,born)
      elseif (pid.eq.6) then
         call cc_bo_1413_2019 (s,born)
      elseif (pid.eq.25) then
         call cc_bo_1314_2122 (s,born)
      elseif (pid.eq.26) then
         call cc_bo_1413_2221 (s,born)
      elseif (pid.eq.27) then
         call cc_bo_2213_1421 (s,born)
      elseif (pid.eq.28) then
         call cc_bo_1322_2114 (s,born)
      elseif (pid.eq.29) then
         call cc_bo_1422_1321 (s,born)
      elseif (pid.eq.30) then
         call cc_bo_2214_2113 (s,born)
      endif

*     DelSV_MS (EW)
*     see Brem_4f_CC.frm [#if `SubtScheme' = 1;*** MSbar subtraction]
      if     ((pid.eq.1).or.(pid.eq.2).or.(pid.eq.3).or.(pid.eq.4).or.
     &        (pid.eq.5).or.(pid.eq.6).or.(pid.eq.25).or.(pid.eq.26)) then
         DelSV_MS=alpha/2/pi*qup**2*
     & (log(Mfscale**2/mup**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mup**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
     &           +alpha/2/pi*qdn**2*
     & (log(Mfscale**2/mdn**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mdn**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
      elseif ((pid.eq.27).or.(pid.eq.28)) then
         DelSV_MS=alpha/2/pi*qup**2*
     & (log(Mfscale**2/mup**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mup**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
     &           +alpha/2/pi*qdn**2*
     & (log(Mfscale**2/mbt**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mbt**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
      elseif ((pid.eq.29).or.(pid.eq.30)) then
         DelSV_MS=alpha/2/pi*qbt**2*
     & (log(Mfscale**2/mbt**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mbt**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
     &           +alpha/2/pi*qdn**2*
     & (log(Mfscale**2/mdn**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mdn**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
      endif
      brdq = DelSV_MS

      xl(1) = (mf+mf1)**2/s
      xu(1) = (s-2d0*sq*omega)/s

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_brdq_r_ew_1s, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_brdq_r_ew_1s, avgi, sd, chi2a)
      brdq = brdq + avgi
      brdq_sd = sd

      return
      end
************************************************************************
      real*8 function function_brdq_r_ew_1s (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 Mfscale,ksi,bornk,brdq
      real*8 sq
      real*8 x(10),wgt
      common /s/ s,sq
      common /Mfscale/ Mfscale
      common /pid/ pid

      ksi = x(1)

      function_brdq_r_ew_1s = 0d0
      if     (pid.eq.1) then
         call cc_bo_1314_1112 (s*ksi,bornk)
      elseif (pid.eq.2) then
         call cc_bo_1413_1211 (s*ksi,bornk)
      elseif (pid.eq.3) then
         call cc_bo_1314_1516 (s*ksi,bornk)
      elseif (pid.eq.4) then
         call cc_bo_1413_1615 (s*ksi,bornk)
      elseif (pid.eq.5) then
         call cc_bo_1314_1920 (s*ksi,bornk)
      elseif (pid.eq.6) then
         call cc_bo_1413_2019 (s*ksi,bornk)
      elseif (pid.eq.25) then
         call cc_bo_1314_2122 (s*ksi,bornk)
      elseif (pid.eq.26) then
         call cc_bo_1413_2221 (s*ksi,bornk)
      elseif (pid.eq.27) then
         call cc_bo_2213_1421 (s*ksi,bornk)
      elseif (pid.eq.28) then
         call cc_bo_1322_2114 (s*ksi,bornk)
      elseif (pid.eq.29) then
         call cc_bo_1422_1321 (s*ksi,bornk)
      elseif (pid.eq.30) then
         call cc_bo_2214_2113 (s*ksi,bornk)
      endif

*     (EW)
*     see Eq.(12) DYCC
      if     ((pid.eq.1).or.(pid.eq.2).or.(pid.eq.3).or.(pid.eq.4).or.
     &        (pid.eq.5).or.(pid.eq.6).or.(pid.eq.25).or.(pid.eq.26)) then
         brdq =
     &        alpha/2/pi*qup**2*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mup**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
     &        +alpha/2/pi*qdn**2*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mdn**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
      elseif ((pid.eq.27).or.(pid.eq.28)) then
         brdq =
     &        alpha/2/pi*qup**2*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mup**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
     &        +alpha/2/pi*qbt**2*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mbt**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
      elseif ((pid.eq.29).or.(pid.eq.30)) then
         brdq =
     &        alpha/2/pi*qbt**2*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mbt**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
     &        +alpha/2/pi*qdn**2*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mdn**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
      endif
      function_brdq_r_ew_1s = brdq
      
      return
      end
************************************************************************
* Hard (1spr), reaction, EW
************************************************************************
      subroutine ProcessHard_R_EW_1spr
     &     (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s,sq
      common /pid/ pid
      external function_hard_r_ew_1spr

      xl(1) = (mf+mf1)**2
      xu(1) = s!-2d0*sq*omega

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_hard_r_ew_1spr, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_hard_r_ew_1spr, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_hard_r_ew_1spr (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 sq
      real*8 x(10),wgt
      common /s/ s,sq
      common /pid/ pid

      spr = x(1)

      function_hard_r_ew_1spr = 0d0
      if     (pid.eq.1) then
         call cc_ha_1314_1112_1spr (s,spr,hard)
      elseif (pid.eq.2) then
         call cc_ha_1413_1211_1spr (s,spr,hard)
      elseif (pid.eq.3) then
         call cc_ha_1314_1516_1spr (s,spr,hard)
      elseif (pid.eq.4) then
         call cc_ha_1413_1615_1spr (s,spr,hard)
      elseif (pid.eq.5) then
         call cc_ha_1314_1920_1spr (s,spr,hard)
      elseif (pid.eq.6) then
         call cc_ha_1413_2019_1spr (s,spr,hard)
c      elseif (pid.eq.25) then
c         call cc_ha_1314_2122_1spr (s,spr,hard)
c      elseif (pid.eq.26) then
c         call cc_ha_1413_2221_1spr (s,spr,hard)
      endif
c      hard = hard*theta_hard(s,spr,omega)
      function_hard_r_ew_1spr = hard
      
      return
      end
************************************************************************
* Hard (4d), reaction, EW
************************************************************************
      subroutine ProcessHard_R_EW(nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s,sq
      common /pid/ pid
      external function_hard_r_ew_4d

      xl(1) = (mf+mf1)**2
      xu(1) = s!-2d0*sq*omega
      xl(2) = 0d0
      xu(2) = 2d0*pi
      xl(3) = -1d0
      xu(3) = +1d0
      xl(4) = -1d0
      xu(4) = +1d0
      
      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (4, function_hard_r_ew_4d, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (4, function_hard_r_ew_4d, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_hard_r_ew_4d (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 sq,hard1,hard2
      real*8 x(10),wgt,ph3,cth3,cth5
      common /pid/ pid
      common /s/ s,sq

      spr  = x(1)
      ph3  = x(2)
      cth3 = x(3)
      cth5 = x(4)

      function_hard_r_ew_4d = 0d0
      if     (pid.eq.1) then
         call cc_ha_1314_1112 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.2) then
         call cc_ha_1413_1211 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.3) then
         call cc_ha_1314_1516 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.4) then
         call cc_ha_1413_1615 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.5) then
         call cc_ha_1314_1920 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.6) then
         call cc_ha_1413_2019 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.25) then
         call cc_ha_1314_2122 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.26) then
         call cc_ha_1413_2221 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.27) then
         call cc_ha_2213_1421 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.28) then
         call cc_ha_1322_2114 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.29) then
         call cc_ha_1422_1321 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.30) then
         call cc_ha_2214_2113 (s,spr,ph3,cth3,cth5,hard)
      endif
c      hard = hard*theta_hard(s,spr,omega)
      function_hard_r_ew_4d = hard

      return
      end
************************************************************************
* Virt+Soft, reaction, QCD
************************************************************************
      subroutine ProcessViSo_R_QCD(nrdm,ncall1,itmx1,itmx2,viso,viso_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 viso,viso_sd,x1cut
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /pid/ pid
      external function_viso_r_qcd_1c

      x1cut = 1d-15
      xl(1) = -1d0+x1cut
      xu(1) = +1d0-x1cut

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_viso_r_qcd_1c, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_viso_r_qcd_1c, avgi, sd, chi2a)
      viso = avgi
      viso_sd = sd

      return
      end
************************************************************************
      real*8 function function_viso_r_qcd_1c (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 sq
      real*8 x(10),wgt
      common /s/ s,sq
      common /pid/ pid

      cosf = x(1)

      if     ((pid.eq.1).or.(pid.eq.2)) then
         t = - s/2*(1-cosf)
         u = - s/2*(1+cosf)
      elseif ((pid.eq.3).or.(pid.eq.4).or.(pid.eq.5).or.(pid.eq.6)) then
         t = - (s-mf**2)/2*(1+cosf)
         u = - (s-mf**2)/2*(1-cosf)
      elseif ((pid.eq.25).or.(pid.eq.26)) then
         t = mbt**2 - 1d0/2*(s+mbt**2-mtp**2)*(1+betaf*cosf)
         u = mbt**2 - 1d0/2*(s+mbt**2-mtp**2)*(1-betaf*cosf)
      elseif ((pid.eq.27).or.(pid.eq.28).or.(pid.eq.29).or.(pid.eq.30)) then
         t =
     &        + (s-mtp**2) * (  - 1d0/2 )
     &        + cosf*(s-mtp**2) * ( + 1d0/2/s*(s-mbt**2) )
     &        + 1d0/2*mbt**2 - 1d0/2*mtp**2*mbt**2/s
         u =
     &        + (s-mtp**2) * (  - 1d0/2 )
     &        + cosf*(s-mtp**2) * ( - 1d0/2/s*(s-mbt**2) )
     &        + 1d0/2*mbt**2 + 1d0/2*mtp**2*mbt**2/s
      endif

      function_viso_r_qcd_1c = 0d0
      if     (pid.eq.1) then
         call cc_qcd_br_1314_1112 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.2) then
         call cc_qcd_br_1413_1211 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.3) then
         call cc_qcd_br_1314_1516 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.4) then
         call cc_qcd_br_1413_1615 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.5) then
         call cc_qcd_br_1314_1920 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.6) then
         call cc_qcd_br_1413_2019 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.25) then
         call cc_qcd_br_1314_2122 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.26) then
         call cc_qcd_br_1413_2221 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.27) then
         call cc_qcd_br_2213_1421 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.28) then
         call cc_qcd_br_1322_2114 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.29) then
         call cc_qcd_br_1422_1321 (-s,-t,-u,born,virt,hard)
      elseif (pid.eq.30) then
         call cc_qcd_br_2214_2113 (-s,-t,-u,born,virt,hard)
      endif
      function_viso_r_qcd_1c = virt

      return
      end
************************************************************************
* Brqd, reaction, QCD
************************************************************************
      subroutine ProcessBrdq_R_QCD(nrdm,ncall1,itmx1,itmx2,brdq,brdq_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 brdq,brdq_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      real*8 DelSV_MS,Mfscale,sq
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s,sq
      common /Mfscale/ Mfscale
      common /pid/ pid
      external function_brdq_r_qcd_1s

      DelSV_MS = 0d0
      brdq     = 0d0
      brdq_sd  = 0d0

      if     (pid.eq.1) then
         call cc_bo_1314_1112 (s,born)
      elseif (pid.eq.2) then
         call cc_bo_1413_1211 (s,born)
      elseif (pid.eq.3) then
         call cc_bo_1314_1516 (s,born)
      elseif (pid.eq.4) then
         call cc_bo_1413_1615 (s,born)
      elseif (pid.eq.5) then
         call cc_bo_1314_1920 (s,born)
      elseif (pid.eq.6) then
         call cc_bo_1413_2019 (s,born)
      elseif (pid.eq.25) then
         call cc_bo_1314_2122 (s,born)
      elseif (pid.eq.26) then
         call cc_bo_1413_2221 (s,born)
      elseif (pid.eq.27) then
         call cc_bo_2213_1421 (s,born)
      elseif (pid.eq.28) then
         call cc_bo_1322_2114 (s,born)
      elseif (pid.eq.29) then
         call cc_bo_1422_1321 (s,born)
      elseif (pid.eq.30) then
         call cc_bo_2214_2113 (s,born)
      endif

*     DelSV_MS (QCD)
      if     ((pid.eq.1).or.(pid.eq.2).or.(pid.eq.3).or.(pid.eq.4).or.
     &        (pid.eq.5).or.(pid.eq.6).or.(pid.eq.25).or.(pid.eq.26)) then
         DelSV_MS=alphas/2/pi*cf*
     & (log(Mfscale**2/mup**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mup**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
     &           +alphas/2/pi*cf*
     & (log(Mfscale**2/mdn**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mdn**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
      brdq = DelSV_MS
      elseif ((pid.eq.27).or.(pid.eq.28)) then
         DelSV_MS=alphas/2/pi*cf*
     & (log(Mfscale**2/mup**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mup**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
     &           +alphas/2/pi*cf*
     & (log(Mfscale**2/mbt**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mbt**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
      brdq = DelSV_MS
      elseif ((pid.eq.29).or.(pid.eq.30)) then
         DelSV_MS=alphas/2/pi*cf*
     & (log(Mfscale**2/mbt**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mbt**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
     &           +alphas/2/pi*cf*
     & (log(Mfscale**2/mdn**2)*log(4d0*omega**2/s)
     &     + 3d0/2*log(Mfscale**2/mdn**2) - log(4d0*omega**2/s) - 3d0/2
     &     - 1d0/2*log(4d0*omega**2/s)**2 + 7d0/2)*born
      brdq = DelSV_MS
      endif

      xl(1) = (mf+mf1)**2/s
      xu(1) = (s-2d0*sq*omega)/s

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_brdq_r_qcd_1s, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_brdq_r_qcd_1s, avgi, sd, chi2a)
      brdq = brdq + avgi
      brdq_sd = sd

      return
      end
************************************************************************
      real*8 function function_brdq_r_qcd_1s (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 Mfscale,ksi,bornk,brdq
      real*8 sq
      real*8 x(10),wgt
      common /s/ s,sq
      common /Mfscale/ Mfscale
      common /pid/ pid

      ksi = x(1)

      function_brdq_r_qcd_1s = 0d0
      if     (pid.eq.1) then
         call cc_bo_1314_1112 (s*ksi,bornk)
      elseif (pid.eq.2) then
         call cc_bo_1413_1211 (s*ksi,bornk)
      elseif (pid.eq.3) then
         call cc_bo_1314_1516 (s*ksi,bornk)
      elseif (pid.eq.4) then
         call cc_bo_1413_1615 (s*ksi,bornk)
      elseif (pid.eq.5) then
         call cc_bo_1314_1920 (s*ksi,bornk)
      elseif (pid.eq.6) then
         call cc_bo_1413_2019 (s*ksi,bornk)
      elseif (pid.eq.25) then
         call cc_bo_1314_2122 (s*ksi,bornk)
      elseif (pid.eq.26) then
         call cc_bo_1413_2221 (s*ksi,bornk)
      elseif (pid.eq.27) then
         call cc_bo_2213_1421 (s*ksi,bornk)
      elseif (pid.eq.28) then
         call cc_bo_1322_2114 (s*ksi,bornk)
      elseif (pid.eq.29) then
         call cc_bo_1422_1321 (s*ksi,bornk)
      elseif (pid.eq.30) then
         call cc_bo_2214_2113 (s*ksi,bornk)
      endif

*     (QCD)
      if     ((pid.eq.1).or.(pid.eq.2).or.(pid.eq.3).or.(pid.eq.4).or.
     &        (pid.eq.5).or.(pid.eq.6).or.(pid.eq.25).or.(pid.eq.26)) then
         brdq =
     &        alphas/2/pi*cf*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mup**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
     &        +alphas/2/pi*cf*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mdn**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
      elseif ((pid.eq.27).or.(pid.eq.28)) then
         brdq =
     &        alphas/2/pi*cf*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mup**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
     &        +alphas/2/pi*cf*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mbt**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
      elseif ((pid.eq.29).or.(pid.eq.30)) then
         brdq =
     &        alphas/2/pi*cf*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mbt**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
     &        +alphas/2/pi*cf*(
     &        (1d0+ksi**2)/(1d0-ksi)*(log(Mfscale**2/mdn**2)
     &        -2d0*log(1d0-ksi)-1d0))
     &        *(bornk)
      endif
      function_brdq_r_qcd_1s = brdq
      
      return
      end
************************************************************************
* Hard (1spr), reaction, QCD
************************************************************************
      subroutine ProcessHard_R_QCD_1spr
     &     (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s,sq
      common /pid/ pid
      external function_hard_r_qcd_1spr

      xl(1) = (mf+mf1)**2
      xu(1) = s!-2d0*sq*omega

      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_hard_r_qcd_1spr, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_hard_r_qcd_1spr, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_hard_r_qcd_1spr (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 sq
      real*8 x(10),wgt
      common /pid/ pid
      common /s/ s,sq

      spr = x(1)

      function_hard_r_qcd_1spr = 0d0

      if     (pid.eq.1) then
         call cc_qcd_ha_1314_1112_1spr (s,spr,hard)
      elseif (pid.eq.2) then
         call cc_qcd_ha_1413_1211_1spr (s,spr,hard)
      elseif (pid.eq.3) then
         call cc_qcd_ha_1314_1516_1spr (s,spr,hard)
      elseif (pid.eq.4) then
         call cc_qcd_ha_1413_1615_1spr (s,spr,hard)
      elseif (pid.eq.5) then
         call cc_qcd_ha_1314_1920_1spr (s,spr,hard)
      elseif (pid.eq.6) then
         call cc_qcd_ha_1413_2019_1spr (s,spr,hard)
      elseif (pid.eq.25) then
         call cc_qcd_ha_1314_2122_1spr (s,spr,hard)
      elseif (pid.eq.26) then
         call cc_qcd_ha_1413_2221_1spr (s,spr,hard)
      endif
c      hard = hard*theta_hard(s,spr,omega)
      function_hard_r_qcd_1spr = hard
      
      return
      end
************************************************************************
* Hard (4d), reaction, QCD
************************************************************************
      subroutine ProcessHard_R_QCD(nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s,sq
      common /pid/ pid
      external function_hard_r_qcd_4d

      xl(1) = (mf+mf1)**2
      xu(1) = s!-2d0*sq*omega
      xl(2) = 0d0
      xu(2) = 2d0*pi
      xl(3) = -1d0
      xu(3) = +1d0
      xl(4) = -1d0
      xu(4) = +1d0
      
      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (4, function_hard_r_qcd_4d, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (4, function_hard_r_qcd_4d, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_hard_r_qcd_4d (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 sq
      real*8 x(10),wgt,ph3,cth3,cth5
      common /pid/ pid
      common /s/ s,sq

      spr  = x(1)
      ph3  = x(2)
      cth3 = x(3)
      cth5 = x(4)

      function_hard_r_qcd_4d = 0d0
      if     (pid.eq.1) then
         call cc_qcd_ha_1314_1112 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.2) then
         call cc_qcd_ha_1413_1211 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.3) then
         call cc_qcd_ha_1314_1516 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.4) then
         call cc_qcd_ha_1413_1615 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.5) then
         call cc_qcd_ha_1314_1920 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.6) then
         call cc_qcd_ha_1413_2019 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.25) then
         call cc_qcd_ha_1314_2122 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.26) then
         call cc_qcd_ha_1413_2221 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.27) then
         call cc_qcd_ha_2213_1421 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.28) then
         call cc_qcd_ha_1322_2114 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.29) then
         call cc_qcd_ha_1422_1321 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.30) then
         call cc_qcd_ha_2214_2113 (s,spr,ph3,cth3,cth5,hard)
      endif
c      hard = hard*theta_hard(s,spr,omega)
      function_hard_r_qcd_4d = hard

      return
      end
************************************************************************
